import os
import time
import pytest as pt
import helics as h


@pt.fixture
def vFed():

    initstring = "-f 1 --name=mainbroker"
    fedinitstring = "--broker=mainbroker --federates=1"
    deltat = 0.01
    # TODO: should add an assert here about helicsGetVersion
    h.helicsGetVersion()

    # Create broker #
    broker = h.helicsCreateBroker("zmq", "", initstring)

    isconnected = h.helicsBrokerIsConnected(broker)

    if isconnected == 1:
        pass

    # Create Federate Info object that describes the federate properties #
    fedinfo = h.helicsCreateFederateInfo()

    # Set Federate name #
    h.helicsFederateInfoSetCoreName(fedinfo, "TestA Core")

    # Set core type from string #
    h.helicsFederateInfoSetCoreTypeFromString(fedinfo, "zmq")

    # Federate init string #
    h.helicsFederateInfoSetCoreInitString(fedinfo, fedinitstring)

    # Set the message interval (timedelta) for federate. Note th#
    # HELICS minimum message time interval is 1 ns and by default
    # it uses a time delta of 1 second. What is provided to the
    # setTimedelta routine is a multiplier for the default timedelta.

    # Set one second message interval #
    h.helicsFederateInfoSetTimeProperty(fedinfo, h.helics_property_time_delta, deltat)

    h.helicsFederateInfoSetIntegerProperty(fedinfo, h.helics_property_int_log_level, 1)

    vFed = h.helicsCreateValueFederate("TestA Federate", fedinfo)

    yield vFed

    h.helicsFederateFinalize(vFed)

    state = h.helicsFederateGetState(vFed)
    assert state == 3

    while h.helicsBrokerIsConnected(broker):
        time.sleep(1)

    h.helicsFederateInfoFree(fedinfo)
    h.helicsFederateFree(vFed)
    h.helicsCloseLibrary()


def test_error_value_federate_initialize():
    initstring = "-f 1 --name=mainbroker"
    with pt.raises(h._helics.HelicsException):
        h.helicsCreateBroker("mq", "", initstring)


def test_value_federate_initialize(vFed):
    state = h.helicsFederateGetState(vFed)
    assert state == 0

    h.helicsFederateEnterExecutingMode(vFed)

    state = h.helicsFederateGetState(vFed)
    assert state == 2


def test_value_federate_publication_registration(vFed):
    pubid1 = h.helicsFederateRegisterTypePublication(vFed, "pub1", "string", "")
    pubid2 = h.helicsFederateRegisterGlobalTypePublication(vFed, "pub2", "int", "")
    pubid3 = h.helicsFederateRegisterTypePublication(vFed, "pub3", "double", "V")
    h.helicsFederateEnterExecutingMode(vFed)

    publication_key = h.helicsPublicationGetKey(pubid1)
    assert publication_key == "TestA Federate/pub1"
    publication_key = h.helicsPublicationGetKey(pubid2)
    assert publication_key == "pub2"
    publication_key = h.helicsPublicationGetKey(pubid3)
    assert publication_key == "TestA Federate/pub3"
    publication_type = h.helicsPublicationGetType(pubid3)
    assert publication_type == "double"
    publication_units = h.helicsPublicationGetUnits(pubid3)
    assert publication_units == "V"


def test_value_federate_runFederateTestNamedPoint(vFed):
    defaultValue = (
        "start of a longer string in place of the shorter one and now this should be very long"
    )
    defVal = 5.3
    # testValue1 = "inside of the functional relationship of helics"
    testValue1 = "short string"
    testVal1 = 45.7823
    testValue2 = "I am a string"
    testVal2 = 0.0

    pubid = h.helicsFederateRegisterGlobalPublication(
        vFed, "pub1", h.helics_data_type_named_point, ""
    )
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")

    h.helicsInputSetDefaultNamedPoint(subid, defaultValue, defVal)

    h.helicsFederateEnterExecutingMode(vFed)

    # publish string1 at time=0.0;
    h.helicsPublicationPublishNamedPoint(pubid, testValue1, testVal1)

    # double val;
    val = h.helicsInputGetNamedPoint(subid)
    # assert value == defaultValue
    assert val == [defaultValue, defVal]

    grantedtime = h.helicsFederateRequestTime(vFed, 1.0)

    assert grantedtime == 0.01

    # get the value
    val2 = h.helicsInputGetNamedPoint(subid)
    # make sure the string is what we expect
    # assert value2 == testValue1
    assert val2 == [testValue1, testVal1]

    # publish a second string
    h.helicsPublicationPublishNamedPoint(pubid, testValue2, testVal2)

    # make sure the value is still what we expect
    val3 = h.helicsInputGetNamedPoint(subid)
    # make sure the string is what we expect
    # assert value3 == testValue1
    assert val3 == [testValue1, testVal1]

    # advance time
    grantedtime = h.helicsFederateRequestTime(vFed, 2.0)
    assert grantedtime == 0.02

    # make sure the value was updated
    val4 = h.helicsInputGetNamedPoint(subid)
    # make sure the string is what we expect
    # assert value4 == testValue2
    assert val4 == [testValue2, testVal2]


def test_value_federate_runFederateTestBool(vFed):
    defaultValue = True
    testValue1 = True
    testValue2 = False

    # register the publications
    pubid = h.helicsFederateRegisterGlobalPublication(vFed, "pub1", h.helics_data_type_boolean, "")
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")

    h.helicsInputSetDefaultBoolean(subid, h.helics_true if defaultValue else h.helics_false)

    h.helicsFederateEnterExecutingMode(vFed)

    # publish string1 at time=0.0;
    h.helicsPublicationPublishBoolean(pubid, h.helics_true if testValue1 else h.helics_false)
    val = h.helicsInputGetBoolean(subid)

    assert val == h.helics_true if defaultValue else h.helics_false

    grantedtime = h.helicsFederateRequestTime(vFed, 1.0)
    assert grantedtime == 0.01

    # get the value
    val = h.helicsInputGetBoolean(subid)

    # make sure the string is what we expect
    assert val == h.helics_true if testValue1 else h.helics_false

    # publish a second string
    h.helicsPublicationPublishBoolean(pubid, h.helics_true if testValue2 else h.helics_false)

    # make sure the value is still what we expect
    val = h.helicsInputGetBoolean(subid)
    assert val == h.helics_true if testValue1 else h.helics_false
    # advance time
    grantedtime = h.helicsFederateRequestTime(vFed, 2.0)
    # make sure the value was updated
    assert grantedtime == 0.02

    val = h.helicsInputGetBoolean(subid)
    assert val == h.helics_false if testValue2 else h.helics_true


def test_value_federate_publisher_registration(vFed):
    pubid1 = h.helicsFederateRegisterPublication(vFed, "pub1", h.helics_data_type_string, "")
    pubid2 = h.helicsFederateRegisterGlobalPublication(vFed, "pub2", h.helics_data_type_int, "")
    pubid3 = h.helicsFederateRegisterPublication(vFed, "pub3", h.helics_data_type_double, "V")
    h.helicsFederateEnterExecutingMode(vFed)

    publication_key = h.helicsPublicationGetKey(pubid1)
    assert publication_key == "TestA Federate/pub1"
    publication_type = h.helicsPublicationGetType(pubid1)
    assert publication_type == "string"
    publication_key = h.helicsPublicationGetKey(pubid2)
    assert publication_key == "pub2"
    publication_key = h.helicsPublicationGetKey(pubid3)
    assert publication_key == "TestA Federate/pub3"
    publication_type = h.helicsPublicationGetType(pubid3)
    assert publication_type == "double"
    publication_units = h.helicsPublicationGetUnits(pubid3)
    assert publication_units == "V"
    publication_type = h.helicsPublicationGetType(pubid2)
    assert publication_type == "int64"


def test_value_federate_subscription_and_publication_registration(vFed):
    pubid3 = h.helicsFederateRegisterTypePublication(vFed, "pub3", "double", "V")

    subid1 = h.helicsFederateRegisterSubscription(vFed, "sub1", "")
    subid2 = h.helicsFederateRegisterSubscription(vFed, "sub2", "")
    subid3 = h.helicsFederateRegisterSubscription(vFed, "sub3", "V")

    h.helicsFederateEnterExecutingMode(vFed)

    publication_type = h.helicsPublicationGetType(pubid3)
    assert publication_type == "double"

    sub_key = h.helicsSubscriptionGetKey(subid1)
    assert sub_key == "sub1"
    sub_type = h.helicsInputGetType(subid1)
    assert sub_type == ""
    sub_key = h.helicsSubscriptionGetKey(subid2)
    assert sub_key == "sub2"
    sub_key = h.helicsSubscriptionGetKey(subid3)
    assert sub_key == "sub3"
    sub_type = h.helicsInputGetType(subid3)
    assert sub_type == ""
    sub_units = h.helicsInputGetUnits(subid3)
    assert sub_units == "V"
    sub_type = h.helicsInputGetType(subid2)
    assert sub_type == ""


def test_value_federate_single_transfer(vFed):

    pubid = h.helicsFederateRegisterGlobalPublication(vFed, "pub1", h.helics_data_type_string, "")
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")

    h.helicsFederateEnterExecutingMode(vFed)

    h.helicsPublicationPublishString(pubid, "string1")

    grantedtime = h.helicsFederateRequestTime(vFed, 1.0)
    assert grantedtime == 0.01

    s = h.helicsInputGetString(subid)
    assert s == "string1"


def test_value_federate_runFederateTestDouble(vFed):
    defaultValue = 1.0
    testValue = 2.0
    pubid = h.helicsFederateRegisterGlobalPublication(vFed, "pub1", h.helics_data_type_double, "")
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")
    h.helicsInputSetDefaultDouble(subid, defaultValue)

    h.helicsFederateEnterExecutingMode(vFed)

    # publish string1 at time=0.0;
    h.helicsPublicationPublishDouble(pubid, testValue)

    value = h.helicsInputGetDouble(subid)
    assert value == defaultValue

    grantedtime = h.helicsFederateRequestTime(vFed, 1.0)
    assert grantedtime == 0.01

    value = h.helicsInputGetDouble(subid)
    assert value == testValue

    # publish string1 at time=0.0;
    h.helicsPublicationPublishDouble(pubid, testValue + 1)

    grantedtime = h.helicsFederateRequestTime(vFed, 2.0)
    assert grantedtime == 0.02

    value = h.helicsInputGetDouble(subid)
    assert value == testValue + 1


def test_value_federate_runFederateTestComplex(vFed):
    rDefaultValue = 1.0
    iDefaultValue = 1.0
    rTestValue = 2.0
    iTestValue = 2.0
    pubid = h.helicsFederateRegisterGlobalPublication(vFed, "pub1", h.helics_data_type_complex, "")
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")
    h.helicsInputSetDefaultComplex(subid, rDefaultValue, iDefaultValue)

    h.helicsFederateEnterExecutingMode(vFed)

    # publish string1 at time=0.0;
    h.helicsPublicationPublishComplex(pubid, rTestValue, iTestValue)

    value1, value2 = h.helicsInputGetComplex(subid)
    assert value1 == rDefaultValue
    assert value2 == iDefaultValue

    grantedtime = h.helicsFederateRequestTime(vFed, 1.0)
    assert grantedtime == 0.01

    value1, value2 = h.helicsInputGetComplex(subid)
    assert value1 == rTestValue
    assert value2 == iTestValue


def test_value_federate_runFederateTestInteger(vFed):
    defaultValue = 1
    testValue = 2
    pubid = h.helicsFederateRegisterGlobalPublication(vFed, "pub1", h.helics_data_type_int, "")
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")
    h.helicsInputSetDefaultInteger(subid, defaultValue)

    h.helicsFederateEnterExecutingMode(vFed)

    h.helicsPublicationPublishInteger(pubid, testValue)

    value = h.helicsInputGetInteger(subid)
    assert value == defaultValue

    grantedtime = h.helicsFederateRequestTime(vFed, 1.0)
    assert grantedtime == 0.01

    value = h.helicsInputGetInteger(subid)
    assert value == testValue

    h.helicsPublicationPublishInteger(pubid, testValue + 1)

    grantedtime = h.helicsFederateRequestTime(vFed, 2.0)
    assert grantedtime == 0.02

    value = h.helicsInputGetInteger(subid)
    assert value == testValue + 1


def test_value_federate_runFederateTestString(vFed):
    defaultValue = "String1"
    testValue = "String2"
    pubid = h.helicsFederateRegisterGlobalPublication(vFed, "pub1", h.helics_data_type_string, "")
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")
    h.helicsInputSetDefaultString(subid, defaultValue)

    h.helicsFederateEnterExecutingMode(vFed)

    h.helicsPublicationPublishString(pubid, testValue)

    value = h.helicsInputGetString(subid)
    assert value == defaultValue

    grantedtime = h.helicsFederateRequestTime(vFed, 1.0)
    assert grantedtime == 0.01

    value = h.helicsInputGetString(subid)
    assert value == testValue


def test_value_federate_runFederateTestVectorD(vFed):
    defaultValue = [0, 1, 2]
    testValue = [3, 4, 5]
    pubid = h.helicsFederateRegisterGlobalPublication(vFed, "pub1", h.helics_data_type_vector, "")
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")
    h.helicsInputSetDefaultVector(subid, defaultValue)

    h.helicsFederateEnterExecutingMode(vFed)

    h.helicsPublicationPublishVector(pubid, testValue)

    value = h.helicsInputGetVector(subid)
    assert value == [0, 1, 2]

    grantedtime = h.helicsFederateRequestTime(vFed, 1.0)
    assert grantedtime == 0.01

    value = h.helicsInputGetVector(subid)
    assert value == [3, 4, 5]


@pt.fixture
def helicsBroker():
    initstring = "-f 1 --name=mainbroker"
    # TODO: should add an assert here about helicsGetVersion
    h.helicsGetVersion()

    # Create broker #
    broker = h.helicsCreateBroker("zmq", "", initstring)

    isconnected = h.helicsBrokerIsConnected(broker)

    if isconnected == 1:
        pass

    yield broker

    h.helicsBrokerFree(broker)
    h.helicsCloseLibrary()


@pt.mark.skipif(
    os.getenv("HELICS_PYTHON_ADDITIONAL_TESTS", "") == "", reason="Disabled by default."
)
def test_value_federate_runFederateTimeoutTest(helicsBroker):

    fedinitstring = "--broker=mainbroker --federates=1"
    deltat = 0.01

    # Create Federate Info object that describes the federate properties #
    fedinfo = h.helicsCreateFederateInfo()

    # Set Federate name #
    h.helicsFederateInfoSetCoreName(fedinfo, "TestA Core")

    # Set core type from string #
    h.helicsFederateInfoSetCoreTypeFromString(fedinfo, "zmq")

    # Federate init string #
    h.helicsFederateInfoSetCoreInitString(fedinfo, fedinitstring)

    # Set the message interval (timedelta) for federate. Note th#
    # HELICS minimum message time interval is 1 ns and by default
    # it uses a time delta of 1 second. What is provided to the
    # setTimedelta routine is a multiplier for the default timedelta.

    # Set one second message interval #
    h.helicsFederateInfoSetTimeProperty(fedinfo, h.helics_property_time_delta, deltat)

    h.helicsFederateInfoSetIntegerProperty(fedinfo, h.helics_property_int_log_level, 1)

    vFed = h.helicsCreateValueFederate("TestA Federate", fedinfo)

    defaultValue = "String1"
    testValue = "String2"
    pubid = h.helicsFederateRegisterGlobalPublication(vFed, "pub1", h.helics_data_type_string, "")
    subid = h.helicsFederateRegisterSubscription(vFed, "pub1", "")
    h.helicsInputSetDefaultString(subid, defaultValue)

    h.helicsFederateEnterExecutingMode(vFed)

    counter = 60
    while counter > 0:
        counter -= 1
        time.sleep(1)

    # Broker should be connected at this point
    assert h.helicsBrokerIsConnected(helicsBroker) == 1, "Broker should still be connected"

    h.helicsFederateFinalize(vFed)

    state = h.helicsFederateGetState(vFed)
    assert state == 3

    while h.helicsBrokerIsConnected(helicsBroker):
        time.sleep(1)

    h.helicsFederateInfoFree(fedinfo)
    h.helicsFederateFree(vFed)



import platform

if platform.system() != "Windows":
    from .helics import *
else:
    try:
        from .helics import *
    except ImportError as e:
        # Python 3 has exception chaining: raise ImportError("Custom message") from e
        # If Python 2 support is dropped, use that instead.
        if 'DLL load failed' in str(e):
            raise ImportError(str(e) + "\nSUGGESTED FIX: Install the latest Visual C++ Redistributable for Visual Studio 2019. See https://support.microsoft.com/en-us/help/2977003/the-latest-supported-visual-c-downloads for links.")
        else:
            raise ImportError(e)

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# HELICS documentation build configuration file, created by
# sphinx-quickstart on Wed Dec 13 12:07:08 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
import os
import sphinx_rtd_theme

current_directory = os.path.dirname(os.path.realpath(__file__))


def which(program):
    def is_exe(fpath):
        return os.path.exists(fpath) and os.access(fpath, os.X_OK) and os.path.isfile(fpath)

    def ext_candidates(fpath):
        yield fpath
        for ext in os.environ.get("PATHEXT", "").split(os.pathsep):
            yield fpath + ext

    # fpath, fname = os.path.split(program)
    fpath = os.path.dirname(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            for candidate in ext_candidates(exe_file):
                if is_exe(candidate):
                    return candidate

    return None


import subprocess

read_the_docs_build = os.environ.get("READTHEDOCS", None) == "True"

if read_the_docs_build:
    dir_name = os.path.realpath(os.path.dirname(__file__))
    subprocess.call("cd {dir_name} && make rtddoxygen".format(dir_name=dir_name), shell=True)
    html_extra_path = [os.path.abspath(os.path.join(dir_name, "../rtd-doxygen"))]

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "sphinx.ext.coverage",
    "sphinx.ext.mathjax",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "sphinx.ext.napoleon",
    "sphinx_markdown_tables",
    "nbsphinx",
    "IPython.sphinxext.ipython_console_highlighting",
    "breathe",
]

from recommonmark.parser import CommonMarkParser
from recommonmark.transform import AutoStructify

breathe_projects = {
    "helics": os.path.abspath(os.path.join(current_directory, "./../build-doxygen/docs/xml")),
}

breathe_default_project = "helics"

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#

source_parsers = {
    ".md": CommonMarkParser,
}
source_suffix = [".rst", ".md"]

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "HELICS"
copyright = "Copyright (c) 2017-2020,\nBattelle Memorial Institute; Lawrence Livermore National Security, LLC; Alliance for Sustainable Energy, LLC.See the top-level NOTICE for additional details.\nAll rights reserved.\nSPDX-License-Identifier: BSD-3-Clause\n"
author = "Philip Top, Jeff Daily, Ryan Mast, Dheepak Krishnamurthy, Andrew Fisher, Himanshu Jain, Bryan Palmintier, Jason Fuller"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = ""
# The full version, including alpha/beta/rc tags.
release = ""

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {"navigation_depth": 4, "collapse_navigation": False}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

html_logo = "./img/HELICS_Logo.png"
html_favicon = html_logo

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# This is required for the alabaster theme
# refs: http://alabaster.readthedocs.io/en/latest/installation.html#sidebars
html_sidebars = {
    "**": [
        "about.html",
        "navigation.html",
        "relations.html",  # needs 'show_related': True theme option to display
        "searchbox.html",
        "donate.html",
    ]
}

# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "HELICSdoc"

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (
        master_doc,
        "HELICS.tex",
        "HELICS Documentation",
        "Philip Top, Jeff Daily, Ryan Mast, Dheepak Krishnamurthy, Andrew Fisher, Himanshu Jain, Bryan Palmintier, Jason Fuller",
        "manual",
    ),
]

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, "helics", "HELICS Documentation", [author], 1)]

# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "HELICS",
        "HELICS Documentation",
        author,
        "HELICS",
        "One line description of project.",
        "Miscellaneous",
    ),
]


def setup(app):
    app.add_stylesheet("css/custom.css")  # may also be an URL
    app.add_config_value("recommonmark_config", {"enable_eval_rst": True,}, True)
    app.add_transform(AutoStructify)



#!/usr/bin/env python

import os
import subprocess
import shlex

CURRENT_DIRECTORY = os.path.realpath(os.path.dirname(__file__))

DEPENDENCIES = os.path.abspath(os.path.join(CURRENT_DIRECTORY, "../dependencies/"))


def get_libraries(executable, substring):

    output = subprocess.check_output(shlex.split("otool -L {}".format(executable))).decode("utf-8")

    print(output)

    libraries = []
    for line in output.splitlines()[1:]:
        line = line.strip()
        if line.startswith("/"):
            continue
        library = line.split(" ")[0]
        if substring in library:
            libraries.append(library)

    return libraries


def fix_install_name(executable, with_rpath=True):

    BOOST_LIBRARIES = get_libraries(executable, "boost")

    for library in BOOST_LIBRARIES:
        cmd = "install_name_tool -change {} @rpath/{} {}".format(
            library, library, executable  # os.path.join(DEPENDENCIES, "boost/lib", library),
        )
        subprocess.call(shlex.split(cmd))

    ZMQ_LIBRARIES = get_libraries(executable, "zmq")

    for library in ZMQ_LIBRARIES:
        cmd = "install_name_tool -change {} @rpath/{} {}".format(
            library, library, executable  # os.path.join(DEPENDENCIES, "zmq/lib", library),
        )
        subprocess.call(shlex.split(cmd))

    if with_rpath is True:

        rpaths = []
        for d in ["zmq", "boost"]:
            rpaths.append("{}".format(os.path.abspath(os.path.join(DEPENDENCIES, d, "lib"))))

        cmd = "install_name_tool -add_rpath {} {}".format(" -add_rpath ".join(rpaths), executable)

        subprocess.call(shlex.split(cmd))


def main(executable_directory=None):

    print("Fixing install names ...")

    if executable_directory is None:
        BUILD_DIRECTORY = os.path.abspath(os.path.join(CURRENT_DIRECTORY, "../build"))
        APPS = os.path.abspath(os.path.join(BUILD_DIRECTORY, "src/helics/apps"))
    else:
        APPS = os.path.abspath(executable_directory)

    for filename in os.listdir(APPS):
        if filename.startswith("helics_") and os.access(
            os.path.abspath(os.path.join(APPS, filename)), os.X_OK
        ):

            print("Fixing for {}".format(filename))
            fix_install_name(os.path.abspath(os.path.join(APPS, filename)))


if __name__ == "__main__":

    import sys

    try:
        executable_directory = sys.argv[1]
    except:
        executable_directory = None
    main(executable_directory)



import time
import pytest as pt
import helics as h


def AddBroker(core_type="zmq", number_of_federates=1):

    initstring = "-f {} --name=mainbroker".format(number_of_federates)

    helicsversion = h.helicsGetVersion()
    print("HELICS version = {}".format(helicsversion))

    # Create broker #
    broker = h.helicsCreateBroker(core_type, "", initstring)

    isconnected = h.helicsBrokerIsConnected(broker)

    if isconnected == 1:
        pass

    return broker


def AddFederate(broker, core_type="zmq", count=1, deltat=1.0, name_prefix="fed"):

    # Create Federate Info object that describes the federate properties #
    fedinfo = h.helicsCreateFederateInfo()

    # Set Federate name #
    h.helicsFederateInfoSetCoreName(fedinfo, name_prefix)

    # Set core type from string #
    h.helicsFederateInfoSetCoreTypeFromString(fedinfo, "zmq")

    # Federate init string #
    fedinitstring = "--broker=mainbroker --federates={}".format(count)
    h.helicsFederateInfoSetCoreInitString(fedinfo, fedinitstring)

    # Set the message interval (timedelta) for federate. Note th#
    # HELICS minimum message time interval is 1 ns and by default
    # it uses a time delta of 1 second. What is provided to the
    # setTimedelta routine is a multiplier for the default timedelta.

    # Set one second message interval #
    h.helicsFederateInfoSetTimeProperty(fedinfo, h.helics_property_time_delta, deltat)

    h.helicsFederateInfoSetIntegerProperty(fedinfo, h.helics_property_int_log_level, 1)

    mFed = h.helicsCreateMessageFederate(name_prefix, fedinfo)

    return mFed, fedinfo


def FreeFederate(fed, fedinfo):
    h.helicsFederateFinalize(fed)
    state = h.helicsFederateGetState(fed)
    assert state == 3  # TODO: should this be 3?

    h.helicsFederateInfoFree(fedinfo)
    h.helicsFederateFree(fed)


@pt.fixture()
def broker():
    broker = AddBroker("zmq", 1)
    yield broker
    h.helicsBrokerDisconnect(broker)
    h.helicsCloseLibrary()


def test_broker_functions(broker):

    initstring = "--broker="
    identifier = h.helicsBrokerGetIdentifier(broker)
    initstring = initstring + identifier
    initstring = initstring + " --broker_address"
    address = h.helicsBrokerGetAddress(broker)
    initstring = initstring + address


def test_message_filter_registration(broker):

    fFed, ffedinfo = AddFederate(broker, "zmq", 1, 1, "filter")
    mFed, mfedinfo = AddFederate(broker, "zmq", 1, 1, "message")

    h.helicsFederateRegisterGlobalEndpoint(mFed, "port1", "")
    h.helicsFederateRegisterGlobalEndpoint(mFed, "port2", None)

    f1 = h.helicsFederateRegisterFilter(fFed, h.helics_filter_type_custom, "filter1")
    f2 = h.helicsFederateRegisterFilter(fFed, h.helics_filter_type_custom, "filter2")
    h.helicsFederateRegisterEndpoint(fFed, "fout", "")
    h.helicsFederateRegisterFilter(fFed, h.helics_filter_type_custom, "filter0/fout")
    h.helicsFederateEnterExecutingModeAsync(fFed)
    h.helicsFederateEnterExecutingMode(mFed)
    h.helicsFederateEnterExecutingModeComplete(fFed)

    filter_name = h.helicsFilterGetName(f1)
    assert filter_name == "filter/filter1"

    filter_name = h.helicsFilterGetName(f2)
    assert filter_name == "filter/filter2"

    # filter_target = h.helicsFilterGetTarget(f2)
    # assert filter_target == "port2"

    h.helicsFederateFinalize(mFed)
    h.helicsFederateFinalize(fFed)

    FreeFederate(fFed, ffedinfo)
    FreeFederate(mFed, mfedinfo)
    time.sleep(1.0)


def test_message_filter_function(broker):

    fFed, ffedinfo = AddFederate(broker, "zmq", 1, 1, "filter")
    mFed, mfedinfo = AddFederate(broker, "zmq", 1, 1, "message")

    p1 = h.helicsFederateRegisterGlobalEndpoint(mFed, "port1", "")
    p2 = h.helicsFederateRegisterGlobalEndpoint(mFed, "port2", "random")

    f1 = h.helicsFederateRegisterGlobalFilter(fFed, h.helics_filter_type_custom, "filter1")
    h.helicsFilterAddSourceTarget(f1, "port1")
    f2 = h.helicsFederateRegisterGlobalFilter(fFed, h.helics_filter_type_delay, "filter2")
    h.helicsFilterAddSourceTarget(f2, "port1")
    h.helicsFederateRegisterEndpoint(fFed, "fout", "")
    f3 = h.helicsFederateRegisterFilter(fFed, h.helics_filter_type_random_delay, "filter3")
    h.helicsFilterAddSourceTarget(f3, "filter/fout")

    h.helicsFilterSet(f2, "delay", 2.5)
    h.helicsFederateEnterExecutingModeAsync(fFed)
    h.helicsFederateEnterExecutingMode(mFed)
    h.helicsFederateEnterExecutingModeComplete(fFed)
    state = h.helicsFederateGetState(fFed)
    assert state == 2
    data = "hello world"

    filt_key = h.helicsFilterGetName(f1)
    assert filt_key == "filter1"

    filt_key = h.helicsFilterGetName(f2)
    assert filt_key == "filter2"

    h.helicsEndpointSendMessageRaw(p1, "port2", data)
    h.helicsFederateRequestTimeAsync(mFed, 1.0)
    grantedtime = h.helicsFederateRequestTime(fFed, 1.0)
    assert grantedtime == 1.0
    grantedtime = h.helicsFederateRequestTimeComplete(mFed)
    assert grantedtime == 1.0
    res = h.helicsFederateHasMessage(mFed)
    assert res == 0
    res = h.helicsEndpointHasMessage(p2)
    assert res == 0
    # grantedtime = h.helicsFederateRequestTime(fFed, 3.0)
    # assert res==h.helics_true

    h.helicsFederateFinalize(mFed)
    h.helicsFederateFinalize(fFed)
    # f2 = h.helicsFederateRegisterDestinationFilter (fFed, h.helics_custom_filter, "filter2", "port2")
    # ep1 = h.helicsFederateRegisterEndpoint (fFed, "fout", "")
    # f3 = h.helicsFederateRegisterSourceFilter (fFed, h.helics_custom_filter, "", "filter0/fout")

    FreeFederate(fFed, ffedinfo)
    FreeFederate(mFed, mfedinfo)
    time.sleep(1.0)



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) 2017-2019,
# Battelle Memorial Institute; Lawrence Livermore National Security, LLC; Alliance for Sustainable Energy, LLC.  See
# the top-level NOTICE for additional details. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

import os
import sys
import github
import git
import shutil
import shlex
import glob
import tarfile
from urllib.parse import urlparse
from subprocess import Popen, PIPE
from git import Repo
from argparse import ArgumentParser, RawTextHelpFormatter
import tempfile

####
# Run a shell command
####


def runcmd(cmd):
    archiveargs = shlex.split(cmd)
    p = Popen(archiveargs, stdout=PIPE, stderr=PIPE)
    stdoutdata, stderrdata = p.communicate()
    return stdoutdata, stderrdata


###
# Send file to github.com
###


def SendFile(FILENAME, CLONE, RELEASE, TOKEN, CLIENTID, CLIENTSECRET):
    REPO = urlparse(CLONE).path
    REPO = REPO.split(".")[0]
    if REPO[0] == "/":
        REPO = REPO[1:]
    REPOSITORY = REPO

    if not os.path.exists(FILENAME):
        print('File "' + FILENAME + '" cannot be found in current directory.')
        sys.exit(1)
    else:
        print("> sending " + FILENAME + "....")

    # Open Github organization/user
    gh = github.Github(
        api_preview=True, login_or_token=TOKEN, client_id=CLIENTID, client_secret=CLIENTSECRET
    )

    # find Repository
    try:
        repo = gh.get_repo(REPOSITORY)
    except github.GithubException as e:
        if e.status == 401:
            print("Bad Credention for repository " + REPOSITORY)
        if e.status == 404:
            print(REPOSITORY + " not found on github")
        sys.exit(1)
    print("> Found " + REPOSITORY)

    # Find release tag (version)
    #
    releases = repo.get_releases()
    found = False
    for i in range(releases.totalCount):
        release = releases.get_page(i)[0]
        if release.tag_name == RELEASE:
            found = True
            break
    if not found:
        print("release " + RELEASE + " not found!")
        sys.exit(1)

    print("> Found " + release.tag_name)

    # Upload file to release tag.
    #
    print("> uploading " + FILENAME + " to " + REPOSITORY + " release " + release.tag_name)
    try:
        release.upload_asset(FILENAME, content_type="application/x-gzip")
    except github.GithubException as e:
        if e.status == 404:
            print("!!  ERROR: check authentication asset not found!")
            sys.exit(1)
        if e.data["errors"][0]["code"] == "already_exists":
            print(">     already exist... patching file...")

        assets = release.get_assets()
        for asset in assets:
            if asset.name == FILENAME:
                asset.update_asset(FILENAME)
    except BaseException as e:
        print("Error, check if file already exist in this directory")
        sys.exit(1)


##
# main program
##
def main():

    # Setup arguments
    DESCRIPTION = """
    Store an extraneous binary file code in the github release page.

    Example:
        . to clone a repository found in current directory
           > git-all-archive.py   [will archive the current tag found in current repository]\n
        . To clone HELICS repository for version v2.1.1
           > git-all-archive.py --clone https://github.com/GMLC-TDC/HELICS.git --tag v2.1.1\n
        . To upload the binary file to a release you can used <GIT_TOKEN> created in your repository setup.
           >  git-all-archive.py --clone https://github.com/dnadeau4/HELICS-src.git  --tag v2.1.1  --token <GIT_TOKEN> --releasename v0.0.0archive\n
        . You can also use <GIT CLIENTID> and <GIT_CLIENTSECRET> if you need to make many upload.
           - <GIT_TOKEN> is limited to 60 request per hour.
           - Using GIT_CLIENTID and GIT_CLIENTSECRET you can increase up to 5000 upload/requests per hour.

    """
    parser = ArgumentParser(description=DESCRIPTION, formatter_class=RawTextHelpFormatter)
    parser.add_argument(
        "--clone",
        action="store",
        dest="CLONE",
        help="clone a github repository [https://github.com/GMLC-TDC/HELICS.git]",
    )
    parser.add_argument(
        "--to_path",
        action="store",
        dest="TOPATH",
        help="clone into a directory [helics_tar_generate",
    )
    parser.add_argument(
        "--releasename",
        action="store",
        dest="RELEASE",
        help="github release/tag to upload if different than version",
    )
    parser.add_argument("--prefix", action="store", dest="PREFIX", help="filename prefix [HELICS]")
    parser.add_argument(
        "--tag", action="store", dest="HELICSTAG", help="github release/tag [v2.1.1]"
    )
    parser.add_argument(
        "--token", action="store", dest="TOKEN", help="github Personal Access Token"
    )
    parser.add_argument(
        "--client_id", action="store", dest="CLIENTID", help="github Client ID Oauth Apps"
    )
    parser.add_argument(
        "--client_secret",
        action="store",
        dest="CLIENTSECRET",
        help="github Client Secret Oauth Apps",
    )

    args = parser.parse_args()

    RELEASE = args.RELEASE
    PREFIX = args.PREFIX
    HELICSTAG = args.HELICSTAG
    TOKEN = args.TOKEN
    CLIENTID = args.CLIENTID
    CLIENTSECRET = args.CLIENTSECRET
    CLONE = args.CLONE
    TOPATH = args.TOPATH
    FILENAME = ""
    # Clone repository into ./HELICS by default
    #
    if CLONE is not None:

        if TOPATH is None:
            TOPATH = "./HELICS"
            if os.path.exists(TOPATH):
                shutil.rmtree(TOPATH)
        Repo.clone_from(CLONE, TOPATH)
        repo = Repo(TOPATH)
    else:
        if os.path.exists("./.git"):
            repo = Repo(".")
            TOPATH = "."
        elif os.path.exists("../.git"):
            repo = Repo("..")
            TOPATH = ".."
        else:
            print('> Current directory does not contain ".git"')
            print('> Are you in "git" repository?')
            print("")
            parser.print_help(sys.stderr)
            sys.exit(1)

    if PREFIX is None:
        PREFIX = "HELICS"

    # Checkout tag or use master
    #
    try:
        if HELICSTAG is None:
            HELICSTAG = repo.active_branch
        else:
            for ref in repo.references:
                if ref.name == HELICSTAG:
                    repo.head.reference = ref
                    HELICSTAG = ref
    except:
        print("> Cannot find " + HELICSTAG)
        print("")
        parser.print_help(sys.stderr)
        sys.exit(1)

    # You can push the tarball in any release tag
    # By default use the HELICSTAG
    if RELEASE is None:
        RELEASE = HELICSTAG.name

    if not (
        isinstance(HELICSTAG, git.refs.tag.TagReference)
        or isinstance(HELICSTAG, git.refs.head.Head)
    ):
        print("Could not find tag " + HELICSTAG + " in repository")
        sys.exit(1)

    # Archive repository and submodules
    #
    os.chdir(TOPATH)  # change directory to clone repository
    print(os.getcwd())
    cmd = "scripts/git-archive.sh " + "-d . " + " -p " + PREFIX + " -n " + HELICSTAG.name
    print(cmd)
    runcmd(cmd)

    # Delete pre-existing work
    #
    TMPDIR = tempfile.mkdtemp()

    print("> creating final file.")
    # Extract all archive tar into /tmp/repo
    filelist = glob.glob("*tar")
    for file in filelist:
        tar = tarfile.open(file)
        tar.extractall(path=TMPDIR)
        tar.close()

    # Create final File  "ex: HELICS-v2.1.1.tar.gz"
    NAME = PREFIX + "-" + HELICSTAG.name
    FILENAME = NAME + ".tar.gz"
    tar = tarfile.open(name=FILENAME, mode="w:gz")
    tar.add(TMPDIR + "/repo", arcname=NAME)
    tar.close()

    # If a github token has been passed try to upload on github.
    # RELEASE can be different tag than VERSION.
    if TOKEN is not None:
        SendFile(FILENAME, CLONE, RELEASE, TOKEN, CLIENTID, CLIENTSECRET)
    else:
        print(FILENAME + " has been created in repository")
        print("without a GitHub Token, you will need to upload it manually.")

    if CLONE:
        shutil.move(FILENAME, "..")

    print(TMPDIR)
    fileList = glob.glob("*.tar")
    # Iterate over the list of filepaths & remove each file.
    for filePath in fileList:
        try:
            os.remove(filePath)
        except OSError as e:
            print("Error while deleting file : ", filePath)

    print("> Done.")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Created on Thu Oct 11 10:08:26 2018

@author: monish.mukherjee
"""
import scipy.io as spio
from pypower.api import case118, ppoption, runpf
import math
import numpy
import matplotlib.pyplot as plt
import time
import helics as h
import random
import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)


def create_broker():
    initstring = "--federates=2 --name=mainbroker"
    broker = h.helicsCreateBroker("zmq", "", initstring)
    isconnected = h.helicsBrokerIsConnected(broker)

    if isconnected == 1:
        pass

    return broker


def create_federate(deltat=1.0, fedinitstring="--federates=1"):

    fedinfo = h.helicsFederateInfoCreate()

    status = h.helicsFederateInfoSetFederateName(fedinfo, "Combination Federate")
    assert status == 0

    status = h.helicsFederateInfoSetCoreTypeFromString(fedinfo, "zmq")
    assert status == 0

    status = h.helicsFederateInfoSetCoreInitString(fedinfo, fedinitstring)
    assert status == 0

    status = h.helicsFederateInfoSetTimeDelta(fedinfo, deltat)
    assert status == 0

    status = h.helicsFederateInfoSetLoggingLevel(fedinfo, 1)
    assert status == 0

    fed = h.helicsCreateCombinationFederate(fedinfo)

    return fed


def destroy_federate(fed):
    h.helicsFederateFinalize(fed)

    #    status, state = h.helicsFederateGetState(fed)
    #    assert state == 3

    while h.helicsBrokerIsConnected(broker):
        time.sleep(1)

    h.helicsFederateFree(fed)
    h.helicsCloseLibrary()


if __name__ == "__main__":

    broker = create_broker()
    # fed = create_federate()

    #################################  Registering  federate from json  ########################################

    fed = h.helicsCreateValueFederateFromConfig("Transmission_json.json")
    h.helicsFederateRegisterInterfaces(fed, "Transmission_json.json")
    federate_name = h.helicsFederateGetName(fed)[-1]
    print(" Federate {} has been registered".format(federate_name))
    pubkeys_count = h.helicsFederateGetPublicationCount(fed)
    subkeys_count = h.helicsFederateGetInputCount(fed)
    print(subkeys_count)
    ######################   Reference to Publications and Subscription form index  #############################
    pubid = {}
    subid = {}
    for i in range(0, pubkeys_count):
        pubid["m{}".format(i)] = h.helicsFederateGetPublicationByIndex(fed, i)
        pubtype = h.helicsPublicationGetType(pubid["m{}".format(i)])
        print(pubtype)
    for i in range(0, subkeys_count):
        subid["m{}".format(i)] = h.helicsFederateGetInputByIndex(fed, i)
        h.helicsInputSetDefaultComplex(subid["m{}".format(i)], 0, 0)
        sub_key = h.helicsSubscriptionGetKey(subid["m{}".format(i)])
        print("Registered Subscription ---> {}".format(sub_key))

    ######################   Entering Execution Mode  ##########################################################
    h.helicsFederateEnterInitializingMode(fed)
    status = h.helicsFederateEnterExecutingMode(fed)

    # Pypower Processing (inputs)
    hours = 24
    total_inteval = int(60 * 60 * hours)
    grantedtime = -1
    pf_interval = 5 * 60  # in seconds (minimim_resolution)
    acopf_interval = 15 * 60  # in seconds (minimim_resolution)
    random.seed(0)

    peak_demand = []
    ppc = []
    case_format = case118()
    peak_demand = case_format["bus"][:, 2][:].copy()
    ppc = case_format.copy()

    ######################   creating fixed load profiles for each bus based on PF interval #############################

    # load profiles (inputs)
    profiles = spio.loadmat(
        "normalized_load_data_1min_ORIGINAL.mat", squeeze_me=True, struct_as_record=False
    )
    load_profiles_1min = profiles["my_data"]
    resolution_load = numpy.floor(total_inteval / pf_interval)
    points = numpy.floor(numpy.linspace(0, len(load_profiles_1min) - 1, resolution_load + 1))
    time_pf = numpy.linspace(0, total_inteval, resolution_load + 1)
    load_profiles = load_profiles_1min[points.astype(int), :]

    ###################   Creating a fixed profile for buses    ##################

    bus_profiles_index = []
    profile_number = 0
    for i in range(len(ppc["bus"])):
        bus_profiles_index.append(profile_number)
        if profile_number == 8:
            profile_number = 0
        else:
            profile_number = profile_number + 1
    ###################   Asserting Profiles to buses    ############################

    # bus_profiles_index = numpy.random.random_integers(0,load_profiles.shape[1]-1,len(ppc['bus']))
    bus_profiles = load_profiles[:, bus_profiles_index]
    time_opf = numpy.linspace(0, total_inteval, numpy.floor(total_inteval / acopf_interval) + 1)

    ###########################   Cosimulation Bus and Load Amplification Factor #########################################

    # Co-sim Bus  (inputs)
    Cosim_bus_number = 118
    cosim_bus = Cosim_bus_number - 1  ## Do not change this line
    load_amplification_factor = 15

    # power_flow
    fig = plt.figure()
    ax1 = fig.add_subplot(2, 1, 1)
    ax2 = fig.add_subplot(2, 1, 2)
    voltage_plot = []
    x = 0
    k = 0
    voltage_cosim_bus = (ppc["bus"][cosim_bus, 7] * ppc["bus"][cosim_bus, 9]) * 1.043

    #########################################   Starting Co-simulation  ####################################################

    for t in range(0, total_inteval, pf_interval):

        ############################   Publishing Voltage to GridLAB-D #######################################################

        voltage_gld = complex(voltage_cosim_bus * 1000)
        logger.info("Voltage value = {} kV".format(abs(voltage_gld) / 1000))
        for i in range(0, pubkeys_count):
            pub = pubid["m{}".format(i)]
            status = h.helicsPublicationPublishComplex(pub, voltage_gld.real, voltage_gld.imag)
        # status = h.helicsEndpointSendEventRaw(epid, "fixed_price", 10, t)

        while grantedtime < t:
            grantedtime = h.helicsFederateRequestTime(fed, t)
        time.sleep(0.1)

        #############################   Subscribing to Feeder Load from to GridLAB-D ##############################################

        for i in range(0, subkeys_count):
            sub = subid["m{}".format(i)]
            rload, iload = h.helicsInputGetComplex(sub)
        logger.info("Python Federate grantedtime = {}".format(grantedtime))
        logger.info("Load value = {} kW".format(complex(rload, iload) / 1000))
        # print(voltage_plot,real_demand)

        actual_demand = peak_demand * bus_profiles[x, :]
        ppc["bus"][:, 2] = actual_demand
        ppc["bus"][:, 3] = actual_demand * math.tan(math.acos(0.85))
        ppc["bus"][cosim_bus, 2] = rload * load_amplification_factor / 1000000
        ppc["bus"][cosim_bus, 3] = iload * load_amplification_factor / 1000000
        ppopt = ppoption(PF_ALG=1)

        print("PF TIme is {} and ACOPF time is {}".format(time_pf[x], time_opf[k]))

        ############################  Running OPF For optimal power flow intervals   ##############################

        if time_pf[x] == time_opf[k]:
            results_opf = runopf(ppc, ppopt)
            if results_opf["success"]:
                ppc["bus"] = results_opf["bus"]
                ppc["gen"] = results_opf["gen"]
                if k == 0:
                    LMP_solved = results_opf["bus"][:, 13]
                else:
                    LMP_solved = numpy.vstack((LMP_solved, results_opf["bus"][:, 13]))
                    opf_time = time_opf[0 : k + 1] / 3600
            k = k + 1

        ################################  Running PF For optimal power flow intervals   ##############################

        solved_pf = runpf(ppc, ppopt)
        results_pf = solved_pf[0]
        ppc["bus"] = results_pf["bus"]
        ppc["gen"] = results_pf["gen"]

        if results_pf["success"] == 1:
            if x == 0:
                voltages = results_pf["bus"][:, 7]
                real_demand = results_pf["bus"][:, 2]
                distribution_load = [rload / 1000000]
            else:
                voltages = numpy.vstack((voltages, results_pf["bus"][:, 7]))
                real_demand = numpy.vstack((real_demand, results_pf["bus"][:, 2]))
                distribution_load.append(rload / 1000000)
                pf_time = time_pf[0 : x + 1] / 3600

            voltage_cosim_bus = results_pf["bus"][cosim_bus, 7] * results_pf["bus"][cosim_bus, 9]
            voltage_plot.append(voltage_cosim_bus)

        ######################### Plotting the Voltages and Load of the Co-SIM bus ##############################################

        if x > 0:
            ax1.clear()
            ax1.plot(pf_time, voltage_plot, "r--")
            ax1.set_xlim([0, 25])
            ax1.set_ylabel("Voltage [in kV]")
            ax1.set_xlabel("Time [in hours]")
            ax2.clear()
            ax2.plot(pf_time, real_demand[:, cosim_bus], "k")
            ax2.set_xlim([0, 25])
            ax2.set_ylabel("Load from distribution [in MW]")
            ax2.set_xlabel("Time [in hours]")
            plt.show(block=False)
            plt.pause(0.1)
        x = x + 1

    ##########################   Creating headers and Printing results to CSVs #####################################

    head = str("Time(in Hours)")
    for i in range(voltages.shape[1]):
        head = head + "," + ("Bus" + str(i + 1))

    numpy.savetxt(
        "Transmission_Voltages.csv",
        numpy.column_stack((pf_time, voltages)),
        delimiter=",",
        fmt="%s",
        header=head,
        comments="",
    )
    numpy.savetxt(
        "Transmission_MW_demand.csv",
        numpy.column_stack((pf_time, real_demand)),
        delimiter=",",
        fmt="%s",
        header=head,
        comments="",
    )
    numpy.savetxt(
        "Transmission_LMP.csv",
        numpy.column_stack((opf_time, LMP_solved)),
        delimiter=",",
        fmt="%s",
        header=head,
        comments="",
    )

    ##############################   Terminating Federate   ########################################################
    t = 60 * 60 * 24
    while grantedtime < t:
        grantedtime = h.helicsFederateRequestTime(fed, t)
    logger.info("Destroying federate")
    destroy_federate(fed)
    logger.info("Done!")



import os

dirname = os.path.realpath(os.path.dirname(__file__))


def get_version():

    with open(os.path.join(dirname, "../CMakeLists.txt")) as f:
        data = f.read()

    for l in data.splitlines():
        if l.startswith("project") and "HELICS VERSION" in l:
            helics_version = l.split()[-1].strip(")")

    return helics_version


def main():

    helics_version = get_version()
    current_source_dir = os.path.abspath(os.path.join(dirname, "../"))
    output_dir = os.path.abspath(os.path.join(dirname, "../build-doxygen/docs"))

    with open(os.path.join(dirname, "../config/Doxyfile.in")) as f:
        doxyfile_template = f.read()

    doxyfile = []

    for l in doxyfile_template.splitlines():
        l = l.replace("@HELICS_VERSION@", helics_version)
        l = l.replace("@CMAKE_CURRENT_SOURCE_DIR@", current_source_dir)
        l = l.replace("@DOXYGEN_OUTPUT_DIR@", output_dir)
        doxyfile.append(l)

    with open(os.path.join(dirname, "../Doxyfile"), "w") as f:
        f.write("\n".join(doxyfile))


if __name__ == "__main__":

    main()



#!/usr/bin/env python3

import os
import subprocess
import shlex

CURRENT_DIRECTORY = os.path.realpath(os.path.dirname(__file__))


def format_file(filename):
    """ Format cmake files """

    cmd = f"cmake-format -i {filename}"
    print(f"Formatting {filename}")
    p = subprocess.Popen(shlex.split(cmd))
    p.__filename = filename
    return p


def main():
    """ cmake-format helper script """

    processes = []

    for r, _, files in os.walk(os.path.join(CURRENT_DIRECTORY, "..")):
        for filename in files:
            if filename == "CMakeLists.txt":
                processes.append(format_file(os.path.abspath(os.path.join(r, filename))))

    for p in processes:
        p.wait()
        if p.returncode != 0:
            print(p.__filename)


if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-
"""
Created on Thu Oct 11 10:08:26 2018

@author: monish.mukherjee
"""
import matplotlib.pyplot as plt
import time
import helics as h
import logging
import pandas as pd


logger = logging.getLogger(__name__)
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)


def destroy_federate(fed):
    h.helicsFederateFinalize(fed)
    h.helicsFederateFree(fed)
    h.helicsCloseLibrary()


if __name__ == "__main__":

    #################################  Registering  federate from json  ########################################

    fed = h.helicsCreateCombinationFederateFromConfig("Control.json")
    federate_name = h.helicsFederateGetName(fed)
    print(federate_name)
    endpoint_count = h.helicsFederateGetEndpointCount(fed)
    subkeys_count = h.helicsFederateGetInputCount(fed)
    print(subkeys_count)
    print(endpoint_count)
    ######################   Reference to Publications and Subscription form index  #############################
    endid = {}
    subid = {}
    for i in range(0, endpoint_count):
        endid["m{}".format(i)] = h.helicsFederateGetEndpointByIndex(fed, i)
        end_name = h.helicsEndpointGetName(endid["m{}".format(i)])
        logger.info("Registered Endpoint ---> {}".format(end_name))

    for i in range(0, subkeys_count):
        subid["m{}".format(i)] = h.helicsFederateGetInputByIndex(fed, i)
        status = h.helicsInputSetDefaultComplex(subid["m{}".format(i)], 0, 0)
        sub_key = h.helicsSubscriptionGetKey(subid["m{}".format(i)])
        logger.info("Registered Subscription ---> {}".format(sub_key))

    print(
        "###############################################################################################"
    )
    print(
        "########################   Entering Execution Mode  ##########################################"
    )
    ######################   Entering Execution Mode  ##########################################################
    h.helicsFederateEnterExecutingMode(fed)

    hours = 24
    total_inteval = int(60 * 60 * hours)
    grantedtime = -1
    update_interval = 5 * 60
    feeder_limit_upper = 4 * (1000 * 1000)
    feeder_limit_lower = 2.7 * (1000 * 1000)
    k = 0
    data = {}
    time_sim = []
    feeder_real_power = []
    feeder_imag_power = []
    for t in range(0, total_inteval, update_interval):

        while grantedtime < t:
            grantedtime = h.helicsFederateRequestTime(fed, t)
        time.sleep(0.1)

        time_sim.append(t / 3600)
        #############################   Subscribing to Feeder Load from to GridLAB-D ##############################################
        key = []
        Real_demand = []
        Imag_demand = []
        for i in range(0, subkeys_count):
            sub = subid["m{}".format(i)]
            rload, iload = h.helicsInputGetComplex(sub)
            sub_key = h.helicsSubscriptionGetKey(sub)
            print(sub_key)
            if "totalLoad" in str(sub_key):
                key_feeder_load = sub_key
                distribution_fed_name = str(key_feeder_load.split("/totalLoad")[0])
                Real_feeder_load = rload
                Imag_feeder_load = iload
                feeder_real_power.append(rload / 1000)
                feeder_imag_power.append(iload / 1000)
            else:
                try:
                    data[sub_key].append(rload / 1000)
                except KeyError:
                    data[sub_key] = [rload / 1000]

                key.append(sub_key)
                Real_demand.append(rload)
                Imag_demand.append(iload)

        logger.info("EV Controller grantedtime = {}".format(grantedtime))

        logger.info("Total Feeder Load is {} + {} j".format(Real_feeder_load, Imag_feeder_load))

        if Real_feeder_load > feeder_limit_upper:
            logger.info("Total Feeder Load is over the Feeder Upper Limit")
            logger.info("Warning ----> Feeder OverLimit --->  Turn off EV")

            if k < endpoint_count:
                end = endid["m{}".format(k)]
                logger.info("endid: {}".format(endid))
                end_name = str(h.helicsEndpointGetName(end))
                logger.info("Sending endpoint name: {}".format(end_name))
                destination_name = end_name.replace(federate_name, distribution_fed_name)
                logger.info(
                    "Endpoint destination: {}".format(h.helicsEndpointGetDefaultDestination(end))
                )
                status = h.helicsEndpointSendMessageRaw(end, "", str("0 + 0 j"))  #
                logger.info("Endpoint sending status: {}".format(status))
                logger.info("Turning off {}".format(end_name))
                k = k + 1
            else:
                logger.info("All EVs are Turned off")

        if Real_feeder_load < feeder_limit_lower:
            logger.info("Total Feeder Load is under the Feeder Lower Limit")
            logger.info("Feeder Can Support EVs ------>  Turn on EV")
            if k > 0:
                k = k - 1
                end = endid["m{}".format(k)]
                end_name = h.helicsEndpointGetName(end)
                destination_name = end_name.replace(federate_name, distribution_fed_name)
                print("Endpoint Destination {}".format(destination_name))
                status = h.helicsEndpointSendMessageRaw(end, "", str("200000 + 0 j"))
                logger.info("Turning on {}".format(end_name))
            else:
                logger.info("All EVs are Turned on")

    fig = plt.figure()
    fig.subplots_adjust(hspace=0.4, wspace=0.4)
    i = 1
    for keys in data:
        ax = fig.add_subplot(2, 3, i)
        ax.plot(time_sim, data[keys])
        ax.set_ylabel("EV Output in kW")
        ax.set_xlabel("Time ")
        ax.set_title(keys)
        i = i + 1

    plt.show(block=True)
    data["time"] = time_sim
    data["feeder_load(real)"] = feeder_real_power
    pd.DataFrame.from_dict(data=data).to_csv("EV_Outputs.csv", header=True)

    t = 60 * 60 * 24
    while grantedtime < t:
        grantedtime = h.helicsFederateRequestTime(fed, t)
    logger.info("Destroying federate")
    destroy_federate(fed)



import io
import os
import sys
import struct
import platform
import subprocess
import versioneer

from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
from distutils import sysconfig
from shutil import copyfile

class HelicsExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)

def _have_swig():
    try:
        subprocess.check_call(['swig', '-version'])
    except OSError:
        return False
    else:
        return True

def _is_vs_cmake_default():
    try:
        cmake_help_output=subprocess.check_output(['cmake', '--help'])
    except OSError:
        return False
    else:
        return '* Visual Studio' in str(cmake_help_output)

class HelicsBuild(build_ext):
    def run(self):
        try:
            subprocess.check_call(['cmake', '--version'])
        except OSError:
            raise RuntimeError("CMake must be installed to build the HELICS extension")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        if not isinstance(ext, HelicsExtension):
            super().build_extension(ext)
            return

        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        helicsdir = os.path.abspath(os.path.join(extdir, 'helics'))
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + helicsdir,
                      '-DCMAKE_BUILD_WITH_INSTALL_RPATH=ON',
                      # For using the new Python find in CMake should set Python3_ROOT_DIR to os.path.split(os.path.dirname(os.path.abspath(sys.executable)))[0]
                      # the CMake FindPython module will likely need changes to support static mode Python interpreters
                      # https://github.com/pypa/manylinux/issues/30 and https://github.com/pypa/manylinux/issues/255
                      #'-DHELICS_USE_NEW_PYTHON_FIND=ON',
                      #'-DPython3_EXECUTABLE=' + sys.executable,
                      #'-DPython3_INCLUDE_DIR=' + sysconfig.get_python_inc(plat_specific=True),
                      #'-DPython3_LIBRARY_RELEASE=/',
                      #'-DPython3_LIBRARY_DEBUG=/',
                     ]

        # This is a hack to get around CMake only working with Python interpreters compiled with --enable-shared
        # Basically Linux+macOS don't require a library to link to, but Windows always requires linking to a library
        # Weird thing: on Windows with multiple installs, setting the Python executable can make it find an incorrect library
        if platform.system() != "Windows":
            cmake_args += ['-DPYTHON_EXECUTABLE=' + sys.executable,
                           '-DPYTHON_LIBRARY=' + os.path.join(sysconfig.get_python_lib(plat_specific=True, standard_lib=True)),
                           '-DPYTHON_INCLUDE_DIR=' + sysconfig.get_python_inc(plat_specific=True),
                          ]
        #else:
        #    cmake_args += ['-DPYTHON_LIBRARY=' + os.path.join(os.path.dirname(sys.executable), 'libs', 'python38.lib'),
        #                   '-DPYTHON_INCLUDE_DIR=' + sysconfig.get_python_inc(plat_specific=True),
        #                  ]

        # Add the Python 2 flag for building a Python 2 compatible swig module
        if sys.version_info[0] == 2:
            cmake_args += ['-DBUILD_PYTHON2_INTERFACE=ON']

        # Use SWIG if it is available
        if _have_swig():
            cmake_args += ['-DHELICS_ENABLE_SWIG=ON']
        else:
            cmake_args += ['-DHELICS_ENABLE_SWIG=OFF']

        # CMake build type
        if self.debug:
            bldcfg = 'Release'
        else:
            bldcfg = 'Release'
        build_args = ['--config', bldcfg]

        # Check if the environment/user set a CMake generator
        user_set_cmake_gen = 'CMAKE_GENERATOR' in os.environ
        
        if platform.system() == "Windows" and ((not user_set_cmake_gen and _is_vs_cmake_default()) or (user_set_cmake_gen and 'Visual Studio' in os.environ.get('CMAKE_GENERATOR'))):
            print("Using VS")
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(bldcfg.upper(), helicsdir)]
            if struct.calcsize('P') == 8:
                cmake_args += ['-A', 'x64']
            else:
                cmake_args += ['-A', 'Win32']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + bldcfg]

        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)

        # Run CMake and build the helics python extension
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=self.build_temp)
        subprocess.check_call(['cmake', '--build', '.'] + build_args, cwd=self.build_temp)

        # Include the helics.py file in the module
        if not os.path.exists(helicsdir):
            os.makedirs(helicsdir)
        copyfile(os.path.join(self.build_temp, 'helics.py'), os.path.join(helicsdir, 'helics.py'))

# Get the directory setup.py is located in
setup_py_dir = os.path.abspath(os.path.dirname(__file__))

# Get the README.md contents
README_contents = io.open(os.path.join(setup_py_dir, 'README.md'), encoding="utf-8").read()

# Use versioneer to get the version tag
PKG_VERSION = versioneer.get_version()

# Try getting a version from PKG-INFO if this is a build from an sdist package
if PKG_VERSION == "0+unknown":
    try:
        with open(os.path.join(setup_py_dir, 'PKG-INFO')) as f:
            for line in f:
                if line.startswith("Version:"):
                    PKG_VERSION=line[8:].strip()
    except IOError:
        print("Unable to get a package version, using 0+unknown")

setup(
    name='helics',
    version=PKG_VERSION,
    author='GMLC-TDC',
    author_email='helicsdevelopers@helics.org',
    maintainer='',
    maintainer_email='',
    url="https://github.com/GMLC-TDC/HELICS",
    download_url="https://github.com/GMLC-TDC/HELICS/releases",
    license='BSD',
    keywords='co-simulation',
    description='Hierarchical Engine for Large-scale Infrastructure Co-Simulation (HELICS)',
    long_description=README_contents,
    long_description_content_type='text/markdown',
    packages=['helics'],
    ext_modules=[HelicsExtension('helics', sourcedir='bundled/helics/interfaces/python')],
    cmdclass={
        'build_ext': HelicsBuild,
        'versioneer': versioneer.get_cmdclass()
    },
    entry_points={
        'console_scripts': [
        ]
    },
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Unix",
        "Programming Language :: C",
        "Programming Language :: C++",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries",
        "Topic :: Scientific/Engineering",
    ],
)
